// CORBAサンプル
// gtk-client for mico

#include "../config.h"

#include <stdio.h>
#include <iostream>
using namespace std;

#include <gtk/gtk.h>
#include <CORBA.h>
#ifdef USE_NAME_SERVICE
#include <mico/CosNaming.h>
#endif

#include "counter.h"
#include "../misc.h"

////////////////////////////////////////////////////////////////////////

struct CounterSet
{
    Counter_var counter;
    GtkWidget* entry;
};

void on_incr(GtkButton* button, CounterSet* cset)
{
    int r;
    try {
        cset->counter->incr();
        r = cset->counter->value();
    }
    catch (...) {
        printf("server lost.\n");
    }
    
    char buf[100];
    sprintf(buf, "%d", r);
    gtk_entry_set_text(GTK_ENTRY(cset->entry), buf);
}

void on_decr(GtkButton* button, CounterSet* cset)
{
    int r;
    try {
        cset->counter->decr();
        r = cset->counter->value();
    }
    catch (...) {
        printf("server lost.\n");
    }
    
    char buf[100];
    sprintf(buf, "%d", r);
    gtk_entry_set_text(GTK_ENTRY(cset->entry), buf);
}

CounterSet csets[2];

GtkWidget* create_gui()
{
    GtkWidget* top = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(top), "gtk+ CORBA client");

    GtkWidget* box = gtk_hbox_new(FALSE, 5);
    gtk_container_set_border_width(GTK_CONTAINER(box), 5);
    gtk_container_add(GTK_CONTAINER(top), box);

    static const char* fr_label[] = {
            "通常カウンタ", "逆算カウンタ" };

    for (int i = 0; i < 2; i++) {
        GtkWidget* frame = gtk_frame_new(fr_label[i]);
        gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 0);

        GtkWidget* fbox = gtk_hbox_new(FALSE, 5);
        gtk_container_set_border_width(GTK_CONTAINER(fbox), 5);
        gtk_container_add(GTK_CONTAINER(frame), fbox);

        csets[i].entry = gtk_entry_new();
        gtk_box_pack_start(GTK_BOX(fbox), csets[i].entry, TRUE, TRUE, 0);

        GtkWidget* incr = gtk_button_new_with_label("incr");
        gtk_widget_set_usize(incr, 50, 0);
        gtk_box_pack_start(GTK_BOX(fbox), incr, FALSE, FALSE, 0);

        GtkWidget* decr = gtk_button_new_with_label("decr");
        gtk_widget_set_usize(decr, 50, 0);
        gtk_box_pack_start(GTK_BOX(fbox), decr, FALSE, FALSE, 0);

        gtk_signal_connect(GTK_OBJECT(incr), "clicked",
                                GTK_SIGNAL_FUNC(on_incr), &csets[i]);
        gtk_signal_connect(GTK_OBJECT(decr), "clicked",
                                GTK_SIGNAL_FUNC(on_decr), &csets[i]);
    }

    gtk_signal_connect(GTK_OBJECT(top), "destroy",
                                GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

    return top;
}

#ifndef USE_NAME_SERVICE
const char* ior_file = "../counter.ior";
#endif

int main(int argc, char* argv[])
{
    gtk_set_locale();
    gtk_init(&argc, &argv);
    
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "mico-local-orb");

#ifdef USE_NAME_SERVICE
    CORBA::Object_var nobj
                    = orb->resolve_initial_references("NameService");
    if (CORBA::is_nil(nobj)) {
        printf("resolve NameService failed.\n");
        return 1;
    }
    CosNaming::NamingContext_var naming
                      = CosNaming::NamingContext::_narrow(nobj);

    CosNaming::Name name;
    name.length(1);
    name[0].id = CORBA::string_dup(COUNTER_NAME);
    name[0].kind = CORBA::string_dup("");
    CORBA::Object_var no = naming->resolve(name);
    csets[0].counter = Counter::_narrow(no);

    name.length(1);
    name[0].id = CORBA::string_dup(REVERSE_NAME);
    name[0].kind = CORBA::string_dup("");
    CORBA::Object_var ro = naming->resolve(name);
    csets[1].counter = ReverseCounter::_narrow(ro);
#else
    FILE* fp = fopen(ior_file, "r");
    if (!fp) {
        perror("file open");
        return 1;
    }
    try {
        CORBA::Object_var obj_nc = orb->string_to_object(read_line(fp));
        CORBA::Object_var obj_rc = orb->string_to_object(read_line(fp));
        fclose(fp);
        if (!obj_nc || !obj_rc) {
            printf("ior error.\n");
            return 1;
        }

        csets[0].counter = Counter::_narrow(obj_nc);
        csets[1].counter = ReverseCounter::_narrow(obj_rc);
    }
    catch (CORBA::Exception& e) {
        e._print(cout);
    }
    catch (...) {
        printf("unknown exception.\n");
    }
#endif  // USE_NAME_SERVICE

    if (CORBA::is_nil(csets[0].counter)
            || CORBA::is_nil(csets[1].counter)) {
        printf("cannot invoke on a nil object\n");
        return 1;
    }

    GtkWidget* top = create_gui();
    gtk_widget_show_all(top);

    gtk_main();

    return 0;
}
