// CORBAץ
// gtk-client for ORBit

#include "../config.h"

#include <stdio.h>
#include <gtk/gtk.h>
#include <orb/orbit.h>
#include <ORBitservices/CosNaming.h>
#include "../misc.h"

#include "counter.h"

CORBA_Environment ev;

struct CounterSet
{
    Counter counter;
    GtkWidget* entry;
};

void on_incr(GtkButton* button, CounterSet* cset)
{
    Counter_incr(cset->counter, &ev);
    int r = Counter__get_value(cset->counter, &ev);

    char buf[100];
    sprintf(buf, "%d", r);
    gtk_entry_set_text(GTK_ENTRY(cset->entry), buf);
}

void on_decr(GtkButton* button, CounterSet* cset)
{
    Counter_decr(cset->counter, &ev);
    int r = Counter__get_value(cset->counter, &ev);

    char buf[100];
    sprintf(buf, "%d", r);
    gtk_entry_set_text(GTK_ENTRY(cset->entry), buf);
}

CounterSet csets[2];

GtkWidget* create_gui()
{
    GtkWidget* top = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(top), "gtk+ CORBA client");

    GtkWidget* box = gtk_hbox_new(FALSE, 5);
    gtk_container_set_border_width(GTK_CONTAINER(box), 5);
    gtk_container_add(GTK_CONTAINER(top), box);

    static const char* fr_label[] = {
            "̾參", "ջ" };

    for (int i = 0; i < 2; i++) {
        GtkWidget* frame = gtk_frame_new(fr_label[i]);
        gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 0);

        GtkWidget* fbox = gtk_hbox_new(FALSE, 5);
        gtk_container_set_border_width(GTK_CONTAINER(fbox), 5);
        gtk_container_add(GTK_CONTAINER(frame), fbox);

        csets[i].entry = gtk_entry_new();
        gtk_box_pack_start(GTK_BOX(fbox), csets[i].entry, TRUE, TRUE, 0);

        GtkWidget* incr = gtk_button_new_with_label("incr");
        gtk_widget_set_usize(incr, 50, 0);
        gtk_box_pack_start(GTK_BOX(fbox), incr, FALSE, FALSE, 0);

        GtkWidget* decr = gtk_button_new_with_label("decr");
        gtk_widget_set_usize(decr, 50, 0);
        gtk_box_pack_start(GTK_BOX(fbox), decr, FALSE, FALSE, 0);

        gtk_signal_connect(GTK_OBJECT(incr), "clicked",
                                GTK_SIGNAL_FUNC(on_incr), &csets[i]);
        gtk_signal_connect(GTK_OBJECT(decr), "clicked",
                                GTK_SIGNAL_FUNC(on_decr), &csets[i]);
    }

    gtk_signal_connect(GTK_OBJECT(top), "destroy",
                                GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

    return top;
}

#ifndef USE_NAME_SERVICE
const char* ior_file = "../counter.ior";
#endif

int main(int argc, char* argv[])
{
    gtk_set_locale();
    gtk_init(&argc, &argv);
    CORBA_exception_init(&ev);
    CORBA_ORB orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);
    
#ifdef USE_NAME_SERVICE
    CosNaming_NamingContext naming
        = CORBA_ORB_resolve_initial_references(orb, "NameService", &ev);
    if (!naming) {
        printf("resolve NameService failed.\n");
        return 1;
    }

    CosNaming_Name* name = CosNaming_Name__alloc();
    name->_length = 1;
    name->_buffer = CORBA_sequence_CosNaming_NameComponent_allocbuf(1);
    name->_buffer->id = CORBA_string_dup("Counter");
    name->_buffer->kind = CORBA_string_dup("");
    csets[0].counter = CosNaming_NamingContext_resolve(naming, name, &ev);

    name = CosNaming_Name__alloc();
    name->_length = 1;
    name->_buffer = CORBA_sequence_CosNaming_NameComponent_allocbuf(1);
    name->_buffer->id = CORBA_string_dup("ReverseCounter");
    name->_buffer->kind = CORBA_string_dup("");
    csets[1].counter = CosNaming_NamingContext_resolve(naming, name, &ev);
#else
    FILE* fp = fopen(ior_file, "r");
    if (!fp) {
        perror("file open");
        return 1;
    }
    csets[0].counter = CORBA_ORB_string_to_object(
                                  orb, read_line(fp), &ev);
    csets[1].counter = CORBA_ORB_string_to_object(
                                  orb, read_line(fp), &ev);
    fclose(fp);
#endif // USE_NAME_SERVICE

    if (!csets[0].counter || !csets[1].counter) {
        printf("ior error.\n");
        return 1;
    }

    GtkWidget* top = create_gui();
    gtk_widget_show_all(top);

    gtk_main();

    CORBA_Object_release(csets[0].counter, &ev);
    CORBA_Object_release(csets[1].counter, &ev);

    return 0;
}
