// server for orbit

#include "../config.h"

#include <stdio.h>
#include <stdlib.h>
#include <orb/orbit.h>

#ifdef USE_NAME_SERVICE
  #include <ORBitservices/CosNaming.h>
#elif defined(USE_OAF)
  #include <liboaf/liboaf.h>
#endif

#include "counter.h"

////////////////////////////////////////////////////////////////////////
// CounterImpl

struct CounterData {
    short value;
    CounterData(short v_ = 0): value(v_) { }
};

#define Counter_DATA(x) ((CounterData*) ((POA_Counter*) (x))->vepv->Counter_epv->_private)

CORBA_short CounterImpl_get_value(PortableServer_Servant servant,
                                  CORBA_Environment* ev) {
    return Counter_DATA(servant)->value;
}

void CounterImpl_incr(PortableServer_Servant servant, CORBA_Environment* ev) {
    Counter_DATA(servant)->value++;
}

void CounterImpl_decr(PortableServer_Servant servant, CORBA_Environment* ev) {
    Counter_DATA(servant)->value--;
}

CORBA_char* CounterImpl_name(PortableServer_Servant servant,
                             CORBA_Environment* ev) {
    return CORBA_string_dup("Counter");
}

void CounterImpl_ctor(POA_Counter__vepv* vepv, int inivalue = 0)
{
    static PortableServer_ServantBase__epv super_epv = { NULL, NULL, NULL };
    static POA_Counter__epv epv = {
        NULL, // Counter饹protectedǡǼ
        CounterImpl_get_value,
        CounterImpl_incr,
        CounterImpl_decr,
        CounterImpl_name };

    vepv->_base_epv = new PortableServer_ServantBase__epv();
    *vepv->_base_epv = super_epv;
    vepv->Counter_epv = new POA_Counter__epv();
    *vepv->Counter_epv = epv;
    vepv->Counter_epv->_private = new CounterData(inivalue);
}

POA_Counter* CounterImpl_new(CORBA_Environment* ev)
{
    POA_Counter* servant = new POA_Counter();
    servant->_private = NULL; // ORBit
    servant->vepv = new POA_Counter__vepv();
    CounterImpl_ctor(servant->vepv);
    POA_Counter__init(servant, ev);
    return servant;
}

void CounterImpl_dtor(POA_Counter__vepv* vepv)
{
    delete ((CounterData*) vepv->Counter_epv->_private);
    delete vepv->Counter_epv;
    delete vepv->_base_epv;
}

void CounterImpl_delete(POA_Counter* servant, CORBA_Environment* ev)
{
    POA_Counter__fini(servant, ev);
    CounterImpl_dtor(servant->vepv);
    delete servant->vepv;
    delete servant;
}

////////////////////////////////////////////////////////////////////////
// ReverseCounterImpl

void ReverseCounterImpl_incr(PortableServer_Servant servant, CORBA_Environment* ev) {
    Counter_DATA(servant)->value--;
}

void ReverseCounterImpl_decr(PortableServer_Servant servant, CORBA_Environment* ev) {
    Counter_DATA(servant)->value++;
}

CORBA_char* ReverseCounterImpl_name(PortableServer_Servant servant,
                                    CORBA_Environment* ev) {
    return CORBA_string_dup("ReverseCounter");
}

void ReverseCounterImpl_ctor(POA_ReverseCounter__vepv* vepv)
{
    static POA_ReverseCounter__epv epv = { NULL };

    CounterImpl_ctor((POA_Counter__vepv*) vepv, 1000);
    vepv->Counter_epv->incr = ReverseCounterImpl_incr; // С饤ɤ
    vepv->Counter_epv->decr = ReverseCounterImpl_decr;
    vepv->Counter_epv->name = ReverseCounterImpl_name;
    vepv->ReverseCounter_epv = new POA_ReverseCounter__epv();
    *vepv->ReverseCounter_epv = epv;
}

POA_ReverseCounter* ReverseCounterImpl_new(CORBA_Environment* ev)
{
    POA_ReverseCounter* servant = new POA_ReverseCounter();
    servant->_private = NULL; // ORBit
    servant->vepv = new POA_ReverseCounter__vepv();
    ReverseCounterImpl_ctor(servant->vepv);
    POA_ReverseCounter__init(servant, ev); // POA_Counter__init()ƤӽФƤ
    return servant;
}

void ReverseCounterImpl_dtor(POA_ReverseCounter__vepv* vepv)
{
    delete vepv->ReverseCounter_epv;
    CounterImpl_dtor((POA_Counter__vepv*) vepv);
}

void ReverseCounterImpl_delete(POA_ReverseCounter* servant, CORBA_Environment* ev)
{
    POA_ReverseCounter__fini(servant, ev); // POA_Counter__fini()ƤӽФƤ
    ReverseCounterImpl_dtor(servant->vepv);
    delete servant->vepv;
    delete servant;
}

//////////////////////////////////////////////////////////////////////

void catchException(CORBA_Environment& ev)
{
    if (ev._major != CORBA_NO_EXCEPTION) {
        printf("CORBA exception: %s\n", CORBA_exception_id(&ev));
        CORBA_exception_free(&ev);
        exit(1);
    }
}

#ifndef USE_NAME_SERVICE
const char* ior_file = "../counter.ior";
#endif

void write_ior(
    FILE* fp,
    CORBA_ORB orb,
    PortableServer_POA poa,
    PortableServer_Servant servant,
    CORBA_Environment* ev)
{
    CORBA_Object obj = PortableServer_POA_servant_to_reference(poa, servant, ev);
    catchException(*ev);
    char* ior = CORBA_ORB_object_to_string(orb, obj, ev);
    catchException(*ev);

    fprintf(fp, "%s\n", ior);

    CORBA_Object_release(obj, ev);
    CORBA_free(ior);
}

void test_ior_file(int argc, char* argv[])
{
    CORBA_Environment ev;
    CORBA_exception_init(&ev);

    // ORB
    CORBA_ORB orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);
    catchException(ev);

    // root POAؤλȤ
    PortableServer_POA poa = (PortableServer_POA)
            CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
    catchException(ev);

    // CORBA֥ȼʥХȡˤ
    POA_Counter* myCounter = CounterImpl_new(&ev);
    catchException(ev);
    POA_ReverseCounter* myReverseCounter = ReverseCounterImpl_new(&ev);
    catchException(ev);

    // ХȤ
    PortableServer_ObjectId* nc_id = 
        PortableServer_POA_activate_object(poa, myCounter, &ev);
    catchException(ev);
    PortableServer_ObjectId* rc_id =
        PortableServer_POA_activate_object(poa, myReverseCounter, &ev);
    catchException(ev);

    // 줾Υ֥ȤIORե˽񤭽Ф
    FILE* fp = fopen(ior_file, "w");
    if (!fp) {
        perror("ior-file open failed");
        exit(1);
    }
    write_ior(fp, orb, poa, myCounter, &ev);
    write_ior(fp, orb, poa, myReverseCounter, &ev);
    fclose(fp);

    // ꥯȤμդ򳫻Ϥ롣
    PortableServer_POAManager poa_manager =
        PortableServer_POA__get_the_POAManager(poa, &ev);
    PortableServer_POAManager_activate(poa_manager, &ev);

    // main loop
    printf("ready.\n");
    CORBA_ORB_run(orb, &ev);

    // 
    PortableServer_POA_deactivate_object(poa, nc_id, &ev);
    PortableServer_POA_deactivate_object(poa, rc_id, &ev);
    CORBA_free(nc_id);
    CORBA_free(rc_id);

    CounterImpl_delete(myCounter, &ev);
    ReverseCounterImpl_delete(myReverseCounter, &ev);

    CORBA_exception_free(&ev);
}
/*
void test_oaf(int argc, char* argv[])
{
    CORBA_ORB orb = oaf_init(argc, argv);

    // Obtain a reference to the root POA.
    PortableServer_POA poa
            = (PortableServer_POA)
            CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
    catchException(ev);

    // We allocate the objects on the heap.  Since these are reference
    // counted objects, they will be deleted by the POA when they are no
    // longer needed.
    POA_Counter* myCounter = CounterImpl_new(&ev);
    catchException(ev);
    POA_ReverseCounter* myReverseCounter = ReverseCounterImpl_new(&ev);
    catchException(ev);

    // Activate the objects.  This tells the POA that the objects are
    // ready to accept requests.
    PortableServer_POA_activate_object(poa, myCounter, &ev);
    catchException(ev);
    PortableServer_POA_activate_object(poa, myReverseCounter, &ev);
    catchException(ev);

    oaf_active_server_register(NC_IID, counter_ref);
    oaf_active_server_register(RC_IID, reverse_ref);

//  2000.11.06 Υɤ㳰ȯ롣
//  oaf_active_server_register()ؤIIDϤʤ
//    oaf_active_server_register("IDL:Counter:1.0", counter_ref);
//    oaf_active_server_register("IDL:ReverseCounter:1.0", reverse_ref);
}

void test_naming(int argc, char* argv[])
{
    CORBA_ORB orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);

    CosNaming_NamingContext naming
            = CORBA_ORB_resolve_initial_references(orb, "NameService", &ev);
    if (!naming) {
        printf("resolve NameService failed.\n");
        return 1;
    }

    CosNaming_Name* name = CosNaming_Name__alloc();
    name->_length = 1;
    name->_buffer = CORBA_sequence_CosNaming_NameComponent_allocbuf(1);
    name->_buffer->id = CORBA_string_dup(COUNTER_NAME);
    name->_buffer->kind = CORBA_string_dup("");
    CosNaming_NamingContext_rebind(naming, name,
            PortableServer_POA_servant_to_reference(poa, nc, &ev), &ev);

    name = CosNaming_Name__alloc();
    name->_length = 1;
    name->_buffer = CORBA_sequence_CosNaming_NameComponent_allocbuf(1);
    name->_buffer->id = CORBA_string_dup(REVERSE_NAME);
    name->_buffer->kind = CORBA_string_dup("");
    CosNaming_NamingContext_rebind(naming, name,
            PortableServer_POA_servant_to_reference(poa, rc, &ev), &ev);
}
*/

int main(int argc, char* argv[])
{
    test_ior_file(argc, argv);
    return 0;
}
/*
#ifdef USE_OAF
    while (true)
        g_main_iteration(TRUE);
#else
#endif

#ifndef USE_NAME_SERVICE
    remove(ior_file);
#endif
    return 0;
}
*/
